#include "AmbientLightControl.h"
#include "Storage.h"

#define MAXBRIGHTNESSVALUE 100
#define MIDBRIGHTNESSVALUE  60
#define HIGHTHRESHOLD  70
#define LOWTHRESHOLD  40
#define BRIGHTNESSHYSTERESIS 10

AmbientLightControl& AmbientLightControl::getInstance()
{
  static AmbientLightControl instance;
  return instance;
}

AmbientLightControl::AmbientLightControl()
{
}

void AmbientLightControl::start()
{
  loadBrightnessAuto(&brightnessAuto);
  loadMinBrightness(&minBrightness);
  loadAmbientMin(&minADC);
  loadAmbientMax(&maxADC);
  lastBrightness = 0;
  adcToBrightness = minBrightness;
  lastadcBrightness = minBrightness;
}

void AmbientLightControl::updateBrightness()
{
  uint8_t currentbrightness = minBrightness;

  if (brightnessAuto)
  {
    currentbrightness = measureAmbientBrightness();
  }

  //only update if necessary
  if ((lastBrightness != currentbrightness))
  {
    //Serial.println("Setting new Brightness");
    ledcontrol.setBrightness(currentbrightness);
    lastBrightness = currentbrightness;
  }
}

void AmbientLightControl::setBrightnessAuto(bool newState)
{
  brightnessAuto = newState;
  saveBrightnessAuto(brightnessAuto);
  updateBrightness();
}

bool AmbientLightControl::getBrightnessAuto(void)
{
  return brightnessAuto;
}

void AmbientLightControl::setMinBrightness(uint8_t newMinBrightness)
{
  minBrightness = newMinBrightness;
  saveMinBrightness(minBrightness);
  updateBrightness();
}

uint8_t AmbientLightControl::getMinBrightness(void)
{
  return minBrightness;
}

uint8_t AmbientLightControl::measureAmbientBrightness(void)
{
  int sensorValue = 0;

  //middle of 10 adc readings
  for (int i = 0; i < 10; i++)
  {
    sensorValue += analogRead(A0);
  }
  sensorValue /= 10;
  // Serial.print("adc average: ");
  // Serial.println(sensorValue);

  /*check for new maxima*/
  if (sensorValue > maxADC)
  {
    maxADC = sensorValue;
    saveAmbientMax(maxADC);
  }
  if (sensorValue < minADC)
  {
    minADC = sensorValue;
    saveAmbientMin(minADC);
  }
  if (minADC == maxADC) {
    maxADC = minADC + 1;
  }
  /*bring adc spectrum to scale*/
  adcToBrightness = map(sensorValue, minADC, maxADC, 0, 100);
  //Serial.print("mapped brightness: ");
  //Serial.println(adcToBrightness);

  /* find new brightness*/
  if (minBrightness == lastadcBrightness)
  {
    if (adcToBrightness > HIGHTHRESHOLD)
    {
      adcToBrightness = MAXBRIGHTNESSVALUE;
      //Serial.println("MAXBRIGHTNESSVALUE 1");
    }
    else if (adcToBrightness > (LOWTHRESHOLD + BRIGHTNESSHYSTERESIS))
    {
      //Serial.println("MIDBRIGHTNESSVALUE 1");
      adcToBrightness = MIDBRIGHTNESSVALUE;
    }
    else
    {
      adcToBrightness = minBrightness;
      //Serial.println("minBrightness 1");
    }
  }

  else if (MIDBRIGHTNESSVALUE == lastadcBrightness)
  {
    if (adcToBrightness < (LOWTHRESHOLD - BRIGHTNESSHYSTERESIS))
    {
      //Serial.println("minBrightness 2");
      adcToBrightness = minBrightness;
    }
    else if (adcToBrightness > (HIGHTHRESHOLD + BRIGHTNESSHYSTERESIS))
    {
      //Serial.println("MAXBRIGHTNESSVALUE 2");
      adcToBrightness = MAXBRIGHTNESSVALUE;
    }
    else
    {
      //Serial.println("MIDBRIGHTNESSVALUE 2");
      adcToBrightness = MIDBRIGHTNESSVALUE;
    }
  }

  else if (MAXBRIGHTNESSVALUE == lastadcBrightness)
  {
    if (adcToBrightness < LOWTHRESHOLD)
    {
      //Serial.println("minBrightness 3");
      adcToBrightness = minBrightness;
    }
    else if (adcToBrightness < (HIGHTHRESHOLD - BRIGHTNESSHYSTERESIS))
    {
      //Serial.println("MIDBRIGHTNESSVALUE 3");
      adcToBrightness = MIDBRIGHTNESSVALUE;
    }
    else
    {
      //Serial.println("MAXBRIGHTNESSVALUE 3");
      adcToBrightness = MAXBRIGHTNESSVALUE;
    }
  }
  else
  {
    //Serial.println("last brightness: err");
    adcToBrightness = minBrightness;
  }
  //Serial.print("brightness to set: ");
  //Serial.println(adcToBrightness);
  lastadcBrightness = adcToBrightness;
  return (uint8_t)adcToBrightness;
}
