#include <Arduino.h>
#include <functional>
#include "WifiControl.h"
#include "BrowserGui.h"
#include "DnsControl.h"
#include "LedControl.h"
#include "Timecontrol.h"
#include "CallbackFunction.h"
#include "AmbientLightControl.h"
#include "Storage.h"
#include <FastLED.h>
#include <fauxmoESP.h>

#define RETRY_TIME 10*60*1000 //10 minutes in milliseconds
#define CWC_VERSION 1.1

bool ClockOn = true;
//if alexa received an order to switch the state
bool SwitchClockState = false;

static TimeControl& timecontrol = TimeControl::getInstance();
static WifiControl& wificontrol = WifiControl::getInstance();
static BrowserGui& browsergui = BrowserGui::getInstance();
static DnsControl& dnscontrol = DnsControl::getInstance();
static LedControl& ledcontrol = LedControl::getInstance();
static AmbientLightControl& ambcontrol = AmbientLightControl::getInstance();
static fauxmoESP fauxmo;
static long lastTry = 0;

/* indicator, if one time has passed, then update leds */
uint8_t last_minutes = 0;
uint8_t last_hours = 0;
uint8_t last_seconds = 0;

void setup()
{
  Serial.begin(115200);
  Serial.println("");
  Serial.print("CWC Version: ");
  Serial.println(CWC_VERSION);

  storageInit();
  ambcontrol.start();
  ledcontrol.start();
  if (!wificontrol.connectWifi())
  {
    if (!wificontrol.connectWifi())
  {
    Serial.println("Save retry time");
    lastTry = millis();
  }
  }

  browsergui.start();
  timecontrol.start();
  dnscontrol.start();

  fauxmo.addDevice("Uhr");
  fauxmo.onSetState([](unsigned char device_id, const char * device_name, bool state)
  {
    if (state)
    {
      ClockOn = true;
    }
    else
    {
      ClockOn = false;
    }
    SwitchClockState = true;
  });

  fauxmo.onGetState([](unsigned char device_id, const char * device_name)
  {
    return ClockOn;
  });

  fauxmo.enable(true);
}

void loop()
{
  uint8_t hours = 0;
  uint8_t minutes = 0;
  uint8_t seconds = 0;

  browsergui.serverLoop();
  dnscontrol.serverLoop();
  fauxmo.handle();
  if (wificontrol.isWifiConnected())
  {
    timecontrol.getTime(&hours, &minutes, &seconds);
    if (last_minutes != minutes || last_hours != hours)
    {
      Serial.print("Time: ");
      Serial.print(hours);
      Serial.print(":");
      Serial.println(minutes);
      last_minutes = minutes;
      last_hours = hours;
      ledcontrol.setTime(hours, minutes);
    }
    if (last_seconds != seconds)
    {
      last_seconds = seconds;
      ambcontrol.updateBrightness();
    }
  }
  else
  {
    Serial.print("No WIFI. Reconnect in ");
    Serial.println((RETRY_TIME - (millis() - lastTry)) / 1000);
    if ((millis() - lastTry) > RETRY_TIME)
    {
      lastTry = millis();
      wificontrol.connectWifi();
    }
  }
  if (true == SwitchClockState)
  {
    SwitchClockState = false;
    if (true == ClockOn)
    {
      ledcontrol.unsetTemporaryColor();
    }
    else
    {
      CRGB newColor = 0;
      ledcontrol.setTemporaryColor(newColor);
    }
  }
}

